using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

using LineDraw; 


namespace XNALinesExample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // For Tab support
        int applicationState = 1; 
        int MaxStates = 3; 

        KeyboardState cKeyboard, pKeyboard;
        MouseState cMouse, pMouse; 
        
        SpriteFont font;
        string output = "";

        float offset = 0.0f;
        float offsetDelta = 0.0f; 
        //-----------
        //Drawing Test
        Rectangle box, box2;
        Circle circle;
        Ellipse ellipse;
       
        Vector2 location = new Vector2(110f, 350f);
        Vector2 location2 = new Vector2(270f, 350f);
        Vector2 EndOffset = Vector2.Zero;
        float ObjectSize = 50;
        float lineOffset = .95f;

        //-----------
        //Intersect Test
        Vector2? intersectA, intersectB, intersectC;
        Line Line1, Line2, Line3;
        Vector2 MouseLoc;
        bool PointLineIntersect = false;

        //-----------
        //Object Test
        bool StepScale = false; 
        bool RotateArrow = false;
        bool MoveOrPoint = true; 
        Arrow arrow;
        Arrow torp; 
        
        
        
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            this.IsMouseVisible = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
            spriteBatch = new SpriteBatch(GraphicsDevice);
            
            box = new Rectangle(25, 25, 100, 200);
            box2 = new Rectangle(350, 100, 100, 200);
            circle = new Circle(new Vector2(650, 200), 100, 32, 4, Color.LawnGreen);
            //ellipse = new Ellipse(new Vector2(0, 0), new Vector2(100, 100), 0, 32, 4, Color.LawnGreen);
            ellipse = new Ellipse(new Vector2(300, 200), new Vector2(100f, 50f));
           
            Line1 = new Line(new Vector2(100, 100), new Vector2(300, 300), 4.0f, Color.Magenta );
            Line2 = new Line(new Vector2(100, 300), new Vector2(300, 100), 4.0f, Color.Cyan);
            Line3 = new Line(new Vector2(100, 100), new Vector2(300, 200), 4.0f, Color.White);

            arrow = new Arrow();
            arrow.Location = new Vector2(300, 300);
            arrow.TranslateLocal(new Vector2 (-20,0)); 
            arrow.ScaleLocal(new Vector2(.4f, .5f));
            
            torp = new Arrow();


            font = Content.Load<SpriteFont>("font"); 
        }
       
        protected override void Update(GameTime gameTime)
        {
            
             cMouse = Mouse.GetState();
             if (pMouse == null) { pMouse = cMouse; }
             cKeyboard = Keyboard.GetState();
             if (pKeyboard == null) { pKeyboard = cKeyboard; }

             // Allows the game to exit w/p X'ing out
             if (cKeyboard.IsKeyDown(Keys.Escape)) { this.Exit(); }

            // Flip between drawing tests and intersection tests
            if (cKeyboard.IsKeyDown(Keys.Tab) && pKeyboard.IsKeyUp(Keys.Tab))
            {
                AdvanceState(); 
            }

            offsetDelta = 120 * (float)gameTime.ElapsedGameTime.TotalSeconds;
            offset = offset + offsetDelta;
            
            switch (applicationState)
            {
                case 1: UpdateIntersectTests(gameTime); break;
                case 2: UpdateDrawingTests(gameTime); break;
                case 3: UpdateObjectTest(gameTime); break;
            }
            
            base.Update(gameTime);
            // Cycle Inputs
            pKeyboard = cKeyboard;
            pMouse = cMouse; 
        }

        public void AdvanceState()
        {
            applicationState++;
            if (applicationState > MaxStates) { applicationState = 1; }
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);
            spriteBatch.Begin();


            switch (applicationState)
            {
                case 1: DrawIntersectTests(gameTime); break;
                case 2: DrawDrawingTests(gameTime); break;
                case 3: DrawObjectTest(gameTime); break; 
            }
           
            spriteBatch.DrawString(font, output, Vector2.Zero, Color.White);

            spriteBatch.End();
            base.Draw(gameTime);
        }

        void UpdateObjectTest(GameTime gameTime)
        {
            // Update What I'm supposed to be doing with the Arrow. 
            if (cKeyboard.IsKeyDown(Keys.LeftControl) && pKeyboard.IsKeyUp(Keys.LeftControl))
            {
                StepScale = !StepScale; 
            }
            if (cKeyboard.IsKeyDown(Keys.LeftAlt) && pKeyboard.IsKeyUp(Keys.LeftAlt))
            {
                RotateArrow = !RotateArrow;
            }
            if (cKeyboard.IsKeyDown(Keys.Space) && pKeyboard.IsKeyUp(Keys.Space))
            {
                MoveOrPoint = !MoveOrPoint;
                if (!MoveOrPoint)
                {
                    torp.Location.X = cMouse.X;
                    torp.Location.Y = cMouse.Y; 
                    RotateArrow = false;
                }
            }

            if (cKeyboard.IsKeyDown(Keys.Q) && pKeyboard.IsKeyUp(Keys.Q))
            {
                arrow.LineColorLocal(Color.Red);
                arrow.LineWidthLocal(5); 
            }
            if (cKeyboard.IsKeyDown(Keys.W) && pKeyboard.IsKeyUp(Keys.W))
            {
                arrow.LineColorLocal(Color.Green);
                arrow.LineWidthLocal(10); 
            }
            if (cKeyboard.IsKeyDown(Keys.E) && pKeyboard.IsKeyUp(Keys.E))
            {
                arrow.LineColorLocal(Color.White);
                arrow.LineWidthLocal(15); 
            }
            if (cKeyboard.IsKeyDown(Keys.R) && pKeyboard.IsKeyUp(Keys.R))
            {
                arrow.LineColorLocal(Color.Blue);
                arrow.LineWidthLocal(8); 
            }



            if (MoveOrPoint)
            {
                // if true, update location
                torp.Location.X = cMouse.X;
                torp.Location.Y = cMouse.Y;
            }
            else
            {
                torp.Roation = LineDrawer.V2ToAngle(new Vector2(cMouse.X, cMouse.Y), torp.Location);
                
            }
            
            if (RotateArrow)
            {
                torp.Roation = offset; 
            }

            if (StepScale)
            {
                if ((cMouse.LeftButton == ButtonState.Pressed) && (pMouse.LeftButton == ButtonState.Released))
                {
                    arrow.Scale = arrow.Scale * 1.25f;
                    //arrow.ScaleLocal(1.25f, 1.25f);
                }

                if ((cMouse.RightButton == ButtonState.Pressed) && (pMouse.RightButton == ButtonState.Released))
                {
                    arrow.Scale = arrow.Scale * .75f;
                    //arrow.ScaleLocal(1.25f, 1.25f);

                }
            }
            else
            {
                if ((cMouse.LeftButton == ButtonState.Pressed))
                {
                    arrow.Scale = arrow.Scale * 1.1f; 
                    //arrow.ScaleLocal(1.1f, 1.1f);
                }

                if ((cMouse.RightButton == ButtonState.Pressed))
                {
                    arrow.Scale = arrow.Scale * .9f; 
                    //arrow.ScaleLocal(.9f, .9f);
                }
            }
        }

        void DrawObjectTest(GameTime gameTime)
        {
            output = "Object Test";
            if (StepScale) { output = output + "\nStep Scaling"; }
            else { output = output + "\nSmooth Scaling"; }

            arrow.Draw(spriteBatch);
            torp.Draw(spriteBatch);
        }


        void UpdateIntersectTests(GameTime gameTime)
        {
            MouseLoc.X = cMouse.X;
            MouseLoc.Y = cMouse.Y; 
            PointLineIntersect = Line3.Intersects(MouseLoc); 
            
            if (cMouse.LeftButton == ButtonState.Pressed)
            {
               
                if (cKeyboard.IsKeyDown(Keys.LeftShift))
                {
                    Line2.startPoint.X = cMouse.X;
                    Line2.startPoint.Y = cMouse.Y;
                }
                else if (cKeyboard.IsKeyDown(Keys.LeftAlt))
                {
                    Line1.startPoint.X = cMouse.X;
                    Line1.startPoint.Y = cMouse.Y;
                }
                else
                {
                    Line3.startPoint.X = cMouse.X;
                    Line3.startPoint.Y = cMouse.Y;
                }
            }
            if (cMouse.RightButton == ButtonState.Pressed)
            {
                if (cKeyboard.IsKeyDown(Keys.LeftShift))
                {
                    Line2.endPoint.X = cMouse.X;
                    Line2.endPoint.Y = cMouse.Y;
                }
                else if (cKeyboard.IsKeyDown(Keys.LeftAlt))
                {
                    Line1.endPoint.X = cMouse.X;
                    Line1.endPoint.Y = cMouse.Y;
                }
                else
                {
                    Line3.endPoint.X = cMouse.X;
                    Line3.endPoint.Y = cMouse.Y;
                }
            }

           intersectA = Line1.Intersect(Line2);
           intersectB = Line3.Intersect(box2);
           intersectC = Line3.Intersect(ellipse);

           ellipse.Roation = offset*4; 

        }

        void DrawIntersectTests(GameTime gameTime)
        {
            output = "Interection Tests";
           // LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Red, Line1.startPoint, 6, 8);
           // LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Red, Line1.endPoint, 6, 8);
           // Line1.Draw(spriteBatch);

           // LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Blue, Line2.startPoint, 6, 8);
           // LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Blue, Line2.endPoint, 6, 8);
           // Line2.Draw(spriteBatch);

           // LineDrawer.DrawRectangle(spriteBatch, 3.0f, Color.White, box2);

            LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Green, Line3.startPoint, 6, 8);
            LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Red, Line3.endPoint, 6, 8);
            Line3.Draw(spriteBatch);

            ellipse.Draw(spriteBatch); 

            if (PointLineIntersect) 
            { 
                //LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Blue, MouseLoc, 6, 16); 
            }
            if (intersectA != null)
            { 
               // LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Green, intersectA.Value, 6, 16); 
            }
            if (intersectB != null)
            { 
                //LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Purple, intersectB.Value, 6, 16); 
            }
            if (intersectC != null)
            { LineDrawer.DrawCircle(spriteBatch, 5.0f, Color.Purple, intersectC.Value, 6, 16); }

           
            
        }



        void UpdateDrawingTests(GameTime gameTime)
        {
           
        }

      

        void DrawDrawingTests(GameTime gameTime)
        {
            output = "Drawing Tests";
            LineDrawer.DrawSolidRectangle(spriteBatch, Color.White, box);
            LineDrawer.DrawSolidRectangle(spriteBatch, Color.Yellow, box, offset);
            LineDrawer.DrawSolidRectangle(spriteBatch, Color.Yellow, box, offset, new Vector2(box.Right, box.Bottom));

            LineDrawer.DrawRectangle(spriteBatch, 3.0f, Color.White, box);
            LineDrawer.DrawRectangle(spriteBatch, 3.0f, Color.Purple, box, offset);
            LineDrawer.DrawRectangle(spriteBatch, 3.0f, Color.Purple, box, offset, new Vector2(box.Right, box.Bottom));

            LineDrawer.DrawEllipse(spriteBatch, 3.0f, Color.Red, box, 32);
            LineDrawer.DrawEllipse(spriteBatch, 3.0f, Color.Green, box, offset, 32);
            LineDrawer.DrawEllipse(spriteBatch, 3.0f, Color.Green, box, offset, new Vector2(box.Right, box.Bottom), 32);

            LineDrawer.DrawCircle(spriteBatch, 3.0f, Color.White, location, 100f, 32);

            LineDrawer.DrawCircle(spriteBatch, 8.0f, Color.White, location, ObjectSize, 32);
            EndOffset = LineDrawer.AngleToV2(offset, ObjectSize * lineOffset);
            LineDrawer.DrawLine(spriteBatch, 4.0f, Color.Yellow, location, location + EndOffset);

            LineDrawer.DrawEllipse(spriteBatch, 8.0f, Color.White, location2, new Vector2(50f, 100f), 32);
            EndOffset = LineDrawer.AngleToEllipseV2(offset, lineOffset, new Vector2(50f, 100f));
            LineDrawer.DrawLine(spriteBatch, 4.0f, Color.Yellow, location2, location2 + EndOffset);

            LineDrawer.DrawEllipse(spriteBatch, 8.0f, Color.Green, location2, new Vector2(50f, 100f), offset, 32);
            LineDrawer.DrawEllipse(spriteBatch, 8.0f, Color.Green, location2, new Vector2(50f, 100f), offset, new Vector2(270f, 250f), 32);


            LineDrawer.DrawArc(spriteBatch, 3.0f, Color.Purple, new Vector2(680, 125), 100f, 25f, 60f, 0f, 6);
            LineDrawer.DrawArc(spriteBatch, 3.0f, Color.Purple, new Vector2(680, 125), 100f, 25f, 60f, 90f, 6);
            LineDrawer.DrawArc(spriteBatch, 3.0f, Color.Purple, new Vector2(680, 125), 100f, 25f, 60f, 180f, 6);
            LineDrawer.DrawArc(spriteBatch, 3.0f, Color.Purple, new Vector2(680, 125), 100f, 25f, 60f, 270f, 6);

            LineDrawer.DrawArc(spriteBatch, 3.0f, Color.Yellow, new Vector2(680, 350), 100f, 25f, 90f, (0f + offset), 6);
            LineDrawer.DrawArc(spriteBatch, 3.0f, Color.Yellow, new Vector2(680, 350), 100f, 25f, 90f, (120f + offset), 6);
            LineDrawer.DrawArc(spriteBatch, 3.0f, Color.Yellow, new Vector2(680, 350), 100f, 25f, 90f, (240f + offset), 6);


            LineDrawer.DrawLine(spriteBatch, 7.0f, Color.Pink, new Vector2(150, 120), new Vector2(250, 220));
            LineDrawer.DrawLine(spriteBatch, 7.0f, Color.Pink, new Vector2(250, 220), new Vector2(350, 120));
            LineDrawer.DrawLine(spriteBatch, 7.0f, Color.Pink, new Vector2(350, 120), new Vector2(250, 020));
            LineDrawer.DrawLine(spriteBatch, 7.0f, Color.Pink, new Vector2(250, 020), new Vector2(150, 120));

            LineDrawer.DrawLine(spriteBatch, 3.0f, Color.Red, new Vector2(150, 120), new Vector2(250, 220));
            LineDrawer.DrawLine(spriteBatch, 3.0f, Color.Red, new Vector2(250, 220), new Vector2(350, 120));
            LineDrawer.DrawLine(spriteBatch, 3.0f, Color.Red, new Vector2(350, 120), new Vector2(250, 020));
            LineDrawer.DrawLine(spriteBatch, 3.0f, Color.Red, new Vector2(250, 020), new Vector2(150, 120));

        }
    }
}
 